part of 'vehicles_on_the_property.dart';

/// [VehicleHistoryScreen] is a StatefulWidget that displays a list of vehicles.

// ignore: must_be_immutable
class VehicleHistoryScreen extends StatefulWidget {
  VehicleHistoryScreen(this.model, {super.key});
  late VehicleModel model;


  @override
  State<VehicleHistoryScreen> createState() => _VehicleHistoryScreenState();
}

class _VehicleHistoryScreenState extends State<VehicleHistoryScreen>
    with Remotable {
  @override
  void initState() {
    super.initState();
    _initializeScrollController();
    _future = _fetch();
  }

  @override
  Widget build(BuildContext context) {
    late final limitedHeaderTextSize =
        LimitedFontSizeUtil.getHeaderFontSize(context);
    late final double limitedBodyTextSize =
        LimitedFontSizeUtil.getBodyFontSize(context);

    return Column(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        if (_hasData == false &&
            _pageNumber <= 1 &&
            _loading == false)
          _buildNoDataFound(context, limitedHeaderTextSize)
        else if (_hasData == true || _pageNumber >= 1)
          _buildHistoryList(context, limitedBodyTextSize),
        if (_hasData == true && _loading == true)
          _buildLoadingIndicator(context),
      ].addToStart(const SizedBox(height: 0)),
    );
  }

  Widget _buildNoDataFound(BuildContext context, double limitedHeaderTextSize) {
    return Expanded(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        mainAxisSize: MainAxisSize.max,
        children: [
          Center(
            child: Text(
              FFLocalizations.of(context).getVariableText(
                ptText: "Nenhum veículo encontrado!",
                enText: "No vehicle found",
              ),
              style: TextStyle(
                fontFamily: 'Nunito',
                fontSize: limitedHeaderTextSize,
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget _buildHistoryList(BuildContext context, double limitedBodyTextSize) {
    return Expanded(
      child: FutureBuilder<List<Widget?>>(
        future: _future,
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.waiting) {
            return Center(child: CircularProgressIndicator());
          } else if (snapshot.hasError) {
            return Center(child: Text('Error: ${snapshot.error}'));
          } else {
            return ListView.builder(
              shrinkWrap: true,
              physics: const BouncingScrollPhysics(),
              controller: _scrollController,
              itemCount: _wrap.length + 1,
              itemBuilder: (context, index) {
                if (index == 0) {
                  return _buildHeader(context, limitedBodyTextSize);
                } else {
                  Widget? item = _wrap[index - 1];
                  return _wrap[index - 1] ?? SizedBox.shrink();
                }
              },
            );
          }
        },
      ),
    );
  }

  Widget _buildHeader(BuildContext context, double limitedBodyTextSize) {
    log('amountRegister: ${widget.model.amountRegister}');
    return Padding(
      padding: const EdgeInsets.only(right: 30, top: 10),
      child: Text(
        (widget.model.amountRegister == '0' ||
                widget.model.amountRegister == null)
            ? ''
            : "${FFLocalizations.of(context).getVariableText(ptText: "Quantidade de Veículos: ", enText: "Amount of Vehicles: ")}${widget.model.amountRegister}/${widget.count}",
        textAlign: TextAlign.right,
        style: TextStyle(
          fontFamily: 'Nunito',
          fontSize: limitedBodyTextSize,
        ),
      ),
    );
  }

  Widget _buildLoadingIndicator(BuildContext context) {
    return Container(
      padding: const EdgeInsets.only(top: 15, bottom: 15),
      child: Center(
        child: CircularProgressIndicator(
          valueColor: AlwaysStoppedAnimation<Color>(
            FlutterFlowTheme.of(context).primary,
          ),
        ),
      ),
    );
  }
}


// Mixin for Fetch Visits
mixin Remotable on State<VehicleHistoryScreen> {
  late Future<List<Widget?>> _future;
  List<Widget?> _wrap = [];
  int _pageNumber = 1;
  bool _hasData = true;
  bool _loading = true;
  int count = 0;


  Future<List<Widget?>> _fetch() async {
    if (!widget._hasData || !widget._loading) return [];

    setState(() => widget._loading = true);

    try {
      var response = await PhpGroup.getVehiclesByProperty.call(widget._pageNumber.toString());
      final List<dynamic> vehicles = response.jsonBody['vehicles'] as List<dynamic>? ?? [];
      safeSetState(() => widget.count = response.jsonBody['total_rows'] ?? 0);

      if (vehicles.isNotEmpty) {
        setState(() {
          
          widget._hasData = true;
          widget._loading = false;
        });

     
        return _generateItems(context, vehicles);
      }

      _showNoMoreDataSnackBar(context);

      setState(() {
        widget._hasData = false;
        widget._loading = false;
      });

      return [];
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed("proccessRequest.php", "", "Consulta de Veículo", e, s);
      setState(() {
        widget._hasData = false;
        widget._loading = false;
      });
      return [];
    }
  }

  void _showNoMoreDataSnackBar(BuildContext context) {
    final message = FFLocalizations.of(context).getVariableText(
      ptText: "Não há mais dados.",
      enText: "No more data.",
    );

    showSnackbar(context, message, true);
  }

  Future<List<Widget?>> _generateItems(BuildContext context, List<dynamic> uItem) async {
    if (!widget._hasData) return [];

    List<Widget?> items = [];

    for (var uItem in uItem) {
      final bool? isOwner = uItem['isOwnerVehicle'];
      final IconData? iconData = isOwner == true ? Symbols.no_crash : Symbols.directions_car;
      final FreCardIcon? cardIcon = isOwner != null
          ? FreCardIcon(height: 50, width: 100, icon: Icon(iconData, size: 80, opticalSize: 10))
          : null;

      final String? tag = uItem['tag'];
      final bool containTag = tag.isNotNullAndEmpty;
      final Map<String, String> labelsHashMap = _generateLabelsHashMap(context, uItem, tag, containTag);

      final statusHashMapList = await _generateStatusHashMapList(uItem);

      Future<void> onTapCardItemAction() async {
        await _handleCardItemTap(context, cardIcon, uItem);
      }

      final statusLinkedHashMap = statusHashMapList.map((map) => LinkedHashMap<String, Color>.from(map)).toList();
      final length = statusLinkedHashMap.expand((e) => [e.length]);

      items.add(CardItemTemplateComponentWidget(
        icon: cardIcon,
        labelsHashMap: labelsHashMap,
        statusHashMap: statusHashMapList,
        onTapCardItemAction: onTapCardItemAction,
        itemWidthFactor: length == 1 ? 0.25 : 0.50,
      ));
    }
    
   widget._wrap.addAll(items);
    return items;
  }

  Map<String, String> _generateLabelsHashMap(BuildContext context,
      Map<String, dynamic> uItem, String? tag, bool containTag) {
    return {
      '${FFLocalizations.of(context).getVariableText(ptText: "Placa", enText: "License Plate")}:':
          uItem['licensePlate'] ?? '',
      '${FFLocalizations.of(context).getVariableText(ptText: "Modelo", enText: "Model")}:':
          uItem['model'] ?? '',
      '${FFLocalizations.of(context).getVariableText(ptText: "Proprietário", enText: "Owner")}:':
          uItem['personName'] ?? '',
      if (containTag)
        '${FFLocalizations.of(context).getVariableText(ptText: "Tag", enText: "Tag")}:':
            tag ?? '',
    };
  }

  Future<List<Map<String, Color>>> _generateStatusHashMapList(
      Map<String, dynamic> uItem) async {
    final statusHashMap =
        await widget.model.generateStatusColorMap(uItem, false);
    return statusHashMap != null ? [statusHashMap] : [];
  }

  Future<void> _handleCardItemTap(BuildContext context, FreCardIcon? cardIcon,
      Map<String, dynamic> uItem) async {
    try {
      final dialogContent = await widget.model.buildVehicleDetails(
        icon: cardIcon,
        item: uItem,
        context: context,
        model: widget.model,
      );

      await showDialog(
        useSafeArea: true,
        context: context,
        builder: (context) =>
            Dialog(alignment: Alignment.center, child: dialogContent),
      ).whenComplete(() {
        safeSetState(() {
          widget._pageNumber = 1;
          widget._wrap = [];
          widget._future = _fetch();
        });
      });
    } catch (e, s) {
      DialogUtil.errorDefault(context);
      LogUtil.requestAPIFailed(
          "proccessRequest.php", "", "Consulta de Veículos", e, s);
      safeSetState(() {
        widget._hasData = false;
        widget._loading = false;
      });
    }
  }

  late ScrollController _scrollController;

  void _initializeScrollController() {
    _scrollController = ScrollController()
      ..addListener(() {
        print('ScrollController');
        if(!widget._hasData) return;
        if (_scrollController.position.pixels >= _scrollController.position.maxScrollExtent) {
          print('ScrollController -> loadMore');
          _loadMore();
        }
      });
  }

  void _loadMore() {
    if (widget._hasData) {
      widget._pageNumber++;
      widget._loading = true;
      widget._future = _fetch();
    }
  }
}
